package com.ruoyi.mingxue.service;


import com.ruoyi.mingxue.domain.MxjbTagExt;

import java.util.List;
import java.util.Map;

/**
 * mxjbTagExtService接口
 * 
 * @author mingxue
 * @date 2022-09-14
 */
public interface IMxjbTagExtService 
{
    /**
     * 查询mxjbTagExt
     * 
     * @param menuId mxjbTagExtID
     * @return mxjbTagExt
     */
    public MxjbTagExt selectMxjbTagExtById(Long menuId);

    /**
     * 查询mxjbTagExt列表
     * 
     * @param mxjbTagExt mxjbTagExt
     * @return mxjbTagExt集合
     */
    public List<MxjbTagExt> selectMxjbTagExtList(MxjbTagExt mxjbTagExt);

    /**
     * 新增mxjbTagExt
     * 
     * @param mxjbTagExt mxjbTagExt
     * @return 结果
     */
    public int insertMxjbTagExt(MxjbTagExt mxjbTagExt);

    /**
     * 修改mxjbTagExt
     * 
     * @param mxjbTagExt mxjbTagExt
     * @return 结果
     */
    public int updateMxjbTagExt(MxjbTagExt mxjbTagExt);

    /**
     * 批量删除mxjbTagExt
     * 
     * @param menuIds 需要删除的mxjbTagExtID
     * @return 结果
     */
    public int deleteMxjbTagExtByIds(Long[] menuIds);

    /**
     * 删除mxjbTagExt信息
     * 
     * @param menuId mxjbTagExtID
     * @return 结果
     */
    public int deleteMxjbTagExtById(Long menuId);

    public Map<Long, Map<String, Object>> buildPcMenuExtMap();
}
