package com.ruoyi.web.controller.system;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.google.common.collect.Lists;
import com.ruoyi.common.core.domain.entity.MenuExtInfo;
import com.ruoyi.mingxue.domain.MxjbTagExt;
import com.ruoyi.mingxue.service.IMxjbTagExtService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysMenu;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.service.ISysMenuService;

/**
 * 菜单信息
 * 
 * @author voluntary
 */
@RestController
@RequestMapping("/system/menu")
public class SysMenuController extends BaseController
{
    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private IMxjbTagExtService mxjbTagExtService;

    /**
     * 获取菜单列表
     */
    @PreAuthorize("@ss.hasPermi('system:menu:list')")
    @GetMapping("/list")
    public AjaxResult list(SysMenu menu)
    {
        List<SysMenu> menus = menuService.selectMenuList(menu, getUserId());
        return success(menus);
    }

    /**
     * 根据菜单编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:menu:query')")
    @GetMapping(value = "/{menuId}")
    public AjaxResult getInfo(@PathVariable Long menuId)
    {
        SysMenu menu = menuService.selectMenuById(menuId);
        List<MxjbTagExt> menuConfList = mxjbTagExtService.selectMxjbTagExtList(new MxjbTagExt());
        if (CollectionUtils.isNotEmpty(menuConfList)) {
            fillAppExt(Lists.newArrayList(menu), menuConfList.stream().collect(Collectors.toMap(MxjbTagExt::getMenuId, MxjbTagExt::getInfo)));
        }
        return AjaxResult.success(menu);
    }

    /**
     * 获取菜单下拉树列表
     */
    @GetMapping("/treeselect")
    public AjaxResult treeselect(SysMenu menu)
    {
        List<SysMenu> menus = menuService.selectMenuList(menu, getUserId());
        return success(menuService.buildMenuTreeSelect(menus));
    }

    /**
     * 加载对应角色菜单列表树
     */
    @GetMapping(value = "/roleMenuTreeselect/{roleId}")
    public AjaxResult roleMenuTreeselect(@PathVariable("roleId") Long roleId)
    {
        List<SysMenu> menus = menuService.selectMenuList(getUserId());
        AjaxResult ajax = AjaxResult.success();
        ajax.put("checkedKeys", menuService.selectMenuListByRoleId(roleId));
        ajax.put("menus", menuService.buildMenuTreeSelect(menus));
        return ajax;
    }

    /**
     * 新增菜单
     */
    @PreAuthorize("@ss.hasPermi('system:menu:add')")
    @Log(title = "菜单管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysMenu menu)
    {
        if (UserConstants.NOT_UNIQUE.equals(menuService.checkMenuNameUnique(menu)))
        {
            return error("新增菜单'" + menu.getMenuName() + "'失败，菜单名称已存在");
        }
        else if (UserConstants.YES_FRAME.equals(menu.getIsFrame()) && !StringUtils.ishttp(menu.getPath()))
        {
            return error("新增菜单'" + menu.getMenuName() + "'失败，地址必须以http(s)://开头");
        }
        menu.setCreateBy(getUsername());
        int iRet = menuService.insertMenu(menu);
        if (null != menu.getConfigExtends() && (null != menu.getConfigExtends().getMeta() || CollectionUtils.isNotEmpty(menu.getConfigExtends().getItems()))) {
            mxjbTagExtService.insertMxjbTagExt(new MxjbTagExt(menu.getMenuId(), menu.getConfigExtends()));
        }
        return toAjax(iRet);    }

    /**
     * 修改菜单
     */
    @PreAuthorize("@ss.hasPermi('system:menu:edit')")
    @Log(title = "菜单管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SysMenu menu)
    {
        if (UserConstants.NOT_UNIQUE.equals(menuService.checkMenuNameUnique(menu)))
        {
            return error("修改菜单'" + menu.getMenuName() + "'失败，菜单名称已存在");
        }
        else if (UserConstants.YES_FRAME.equals(menu.getIsFrame()) && !StringUtils.ishttp(menu.getPath()))
        {
            return error("修改菜单'" + menu.getMenuName() + "'失败，地址必须以http(s)://开头");
        }
        else if (menu.getMenuId().equals(menu.getParentId()))
        {
            return error("修改菜单'" + menu.getMenuName() + "'失败，上级菜单不能选择自己");
        }
        menu.setUpdateBy(getUsername());
        int iRet = menuService.updateMenu(menu);
        MxjbTagExt exist = mxjbTagExtService.selectMxjbTagExtById(menu.getMenuId());
        if (null != menu.getConfigExtends() && (null != menu.getConfigExtends().getMeta() || CollectionUtils.isNotEmpty(menu.getConfigExtends().getItems()))) {
            if (null != exist) {
                mxjbTagExtService.updateMxjbTagExt(new MxjbTagExt(menu.getMenuId(), menu.getConfigExtends()));
            } else {
                mxjbTagExtService.insertMxjbTagExt(new MxjbTagExt(menu.getMenuId(), menu.getConfigExtends()));
            }
        } else if(null != exist) {
            mxjbTagExtService.deleteMxjbTagExtById(menu.getMenuId());
        }
        return toAjax(iRet);    }

    /**
     * 删除菜单
     */
    @PreAuthorize("@ss.hasPermi('system:menu:remove')")
    @Log(title = "菜单管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{menuId}")
    public AjaxResult remove(@PathVariable("menuId") Long menuId)
    {
        if (menuService.hasChildByMenuId(menuId))
        {
            return warn("存在子菜单,不允许删除");
        }
        if (menuService.checkMenuExistRole(menuId))
        {
            return warn("菜单已分配,不允许删除");
        }
        mxjbTagExtService.deleteMxjbTagExtById(menuId);
        return toAjax(menuService.deleteMenuById(menuId));
    }

    private List<SysMenu> fillAppExt(List<SysMenu> menus, Map<Long, MenuExtInfo.ConfigExtends> appMenuExtMap) {
        menus.forEach(t -> {
            t.setConfigExtends(appMenuExtMap.get(t.getMenuId()));
            if (CollectionUtils.isNotEmpty(t.getChildren())) {
                fillAppExt(t.getChildren(), appMenuExtMap);
            }
        });
        return menus;
    }
}